==> mutual_credit-5.0.12.tar.gz: mutual_credit/modules/signatures/src/SignatureService.php <==
<?php

namespace Drupal\mcapi_signatures;

use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\mysql\Driver\Database\mysql\Connection;
use Drupal\mcapi\Plugin\TransactionRelativeManager;
use Drupal\mcapi\Entity\Transaction;
use Drupal\user\Entity\User;
use Drupal\mailsystem\MailsystemManager;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Component\Datetime\Time;

/**
 * Signature service.
 *
 * Loads, signs and gives info relating to transaction signatures.
 */
class SignatureService {

  /**
   * @var TransactionInterface
   */
  private $transaction;

  /**
   * @var AccountInterface
   */
  private $currentUser;

  /**
   * @var Connection
   */
  private $database;

  /**
   * @var int
   */
  private $requestTime;

  /**
   * @var TransactionRelativeManager
   */
  private $transactionRelativeManager;

  /**
   *
   * @var Drupal\Core\Config\ImmutableConfig
   */
  private $config;
  /**
   *
   * @var Drupal\Core\Logger\LoggerChannel
   */
  private $logger;
  /**
   *
   * @var Drupal\mailsystem\MailsystemManager
   */
  private $mail_manager;

  /**
   * @param AccountInterface $current_user
   * @param Connection $database
   * @param TransactionRelativeManager $transaction_relatives
   * @param Time $time
   */
  public function __construct(AccountInterface $current_user, Connection $database, TransactionRelativeManager $transaction_relatives, Time $time, ConfigFactory $config_factory, LoggerChannelFactory $logger_factory, MailsystemManager $mail_manager) {
    $this->currentUser = $current_user;
    $this->database = $database;
    $this->transactionRelativeManager = $transaction_relatives;
    $this->requestTime = $time->getRequestTime();
    $this->config = $config_factory->get('mcapi_signatures.settings');
    $this->logger = $logger_factory->get('mcapi');
    $this->mailManager = $mail_manager;
  }

  /**
   * Initialise this service by telling it which transaction to work on.
   */
  public function setTransaction(TransactionInterface $transaction) : SignatureService {
    $this->transaction = $transaction;
    return $this;
  }
  /**
   * Return the transaction
   *
   * @param TransactionInterface $transaction
   * @return Transaction
   */
  public function getTransaction() : Transaction {
    return $this->transaction;
  }

  /**
   * Respond to the transaction being deleted.
   *
   * Delete from the signatory table.
   */
  public function delete() {
    $this->database->delete('mcapi_signatures')
      ->condition('serial', $this->transaction->serial->value)
      ->execute();
  }

  /**
   * Respond to the transaction being inserted.
   *
   * Write the signatories to the signatory table.
   */
  public function insert() : SignatureService {
    $q = $this->database->insert('mcapi_signatures')
      ->fields(['serial', 'uid', 'signed']);
    foreach ($this->transaction->signatures as $uid => $signed_unixtime) {
      $q->values([$this->transaction->serial->value, $uid, $signed_unixtime]);
    }
    $q->execute();
    $this->notify();
    return $this;
  }

  /**
   * Respond to the transaction being updated.
   *
   * Update the signatory table.
   */
  public function update() {
    // New signatures cannot be added, only existing ones changed.
    $serial = $this->transaction->serial->value;
    foreach ($this->transaction->signatures as $uid => $signed) {
      $this->database->update('mcapi_signatures')
        ->fields(['signed' => $signed])
        ->condition('serial', $serial)
        ->condition('uid', $uid)
        ->execute();
    }
  }

  /**
   * Add signatures to the transaction according to the configured relatives.
   */
  public function addSignatures() : SignatureService {
    $relatives = $this->transaction->type->entity
      ->getThirdPartySetting('mcapi_signatures', 'relations', []);
    $user_ids = $this->transactionRelativeManager->getUsers($this->transaction, $relatives);
    foreach ($user_ids as $uid) {
      $this->transaction->signatures[$uid] = 0;
    }
    return $this;
  }

  /**
   * Sign a transaction if the users signature is required.
   *
   * Change the state if no more signatures are left.
   *
   * @param int $uid
   *   ID of the user who is signing - defaults to the current user
   */
  public function sign($uid = NULL) {
    if (is_null($uid)) {
      $uid = $this->currentUser->id();
    }
    if (isset($this->transaction->signatures[$uid]) and empty($this->transaction->signatures[$uid])) {
      $this->transaction->signatures[$uid] = $this->requestTime;
      // Set the state to finished if there are no outstanding signatures.
      if (array_search(0, $this->transaction->signatures) === FALSE) {
        $this->transaction->set('state', 'done');
        // This should affect all the children
      }
    }
  }

  /**
   * Fulfil all remaining signatures on the transaction (admin only).
   */
  public function signOff() {
    foreach ($this->transaction->signatures as $uid => $signed) {
      if (!$signed) {
        $this->transaction->signatures[$uid] = $this->requestTime;
      }
    }
    $this->transaction->set('state', 'done');
  }

  /**
   * Determine which users are due to sign.
   *
   * @param int $uid
   *   The user ID.
   *
   * @return integer[]
   *   IDs of users.
   */
  public function waitingOn($uid = NULL) : array {
    if (is_null($uid)) {
      $uid = $this->currentUser->id();
    }
    $uids = [];
    if ($this->transaction->state->target_id == 'pending') {
      if (empty($this->transaction->signatures)) {
        trigger_error('Invalid pending transaction '.$this->transaction->serial->value.' has no signatures.', E_USER_ERROR);
      }
      foreach ($this->transaction->signatures as $user_id => $signed) {
        if (!$signed) {
          $uids[] = $user_id;
        }
      }
    }
    return $uids;
  }

  /**
   * Determine whether the given user's signature is needed.
   *
   * @param int $uid
   *   The user ID.
   *
   * @return bool
   *   TRUE if the transaction needs the user's signature
   */
  public function isWaitingOn($uid = NULL) : bool {
    if (is_null($uid)) {
      $uid = $this->currentUser->id();
    }
    $uids = [];
    if ($this->transaction->state->target_id == 'pending') {
      foreach ($this->transaction->signatures as $user_id => $signed) {
        if ($user_id == $uid and !$signed) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
   * Email users who need to sign.
   */
  function notify() {
    $transaction = $this->getTransaction();
    if (in_array($transaction->type->target_id , (array)$this->config->get('types'))) {
      foreach ($this->waitingOn() as $uid) {
        $account = User::load($uid);
        $message = $this->mailManager->mail(
          'mcapi_signatures',
          'signhere',
          $account->getEmail(),
          $account->getPreferredLangcode(),
          ['recipient' => $account, 'transaction' => $transaction]
        );
        if ($message['result']) {
          $this->logger->notice(
            'Mailed %name to sign transaction @link',
            ['%name' => $account->getDisplayName(), '@link' => $transaction->toLink()->toString()]
          );
        }
      }
    }
  }

  /**
   * Get the transactions needing signature of a given user.
   *
   * @param int $uid
   *   The user ID.
   *
   * @return int[]
   *   Serial numbers of the transactions.
   *
   * @todo this whole module needs to handle the database better, maybe by
   * decorating the Transaction or TransactionStorage.
   */
  public static function transactionsNeedingSigOfUser($uid) : array {
    // Assumes all transactions referenced are in pending state.
    return \Drupal::database()->select("mcapi_signatures", 's')
      ->fields('s', ['serial'])
      ->condition('uid', $uid)
      ->condition('signed', 0)
      ->execute()
      ->fetchCol();
  }

}

==> mutual_credit-5.1.0.tar.gz: mutual_credit/modules/signatures/src/SignatureService.php <==
<?php

namespace Drupal\mcapi_signatures;

use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\mysql\Driver\Database\mysql\Connection;
use Drupal\mcapi\Plugin\TransactionRelativeManager;
use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Component\Datetime\Time;

/**
 * Signature service.
 *
 * Loads, signs and gives info relating to transaction signatures.
 */
class SignatureService {

  /**
   * @var TransactionInterface
   */
  private $transaction;

  /**
   * @var AccountInterface
   */
  private $currentUser;

  /**
   * @var Connection
   */
  private $database;

  /**
   * @var int
   */
  private $requestTime;

  /**
   * @var TransactionRelativeManager
   */
  private $transactionRelativeManager;

  /**
   *
   * @var Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * @param AccountInterface $current_user
   * @param Connection $database
   * @param TransactionRelativeManager $transaction_relatives
   * @param Time $time
   */
  public function __construct(AccountInterface $current_user, Connection $database, TransactionRelativeManager $transaction_relatives, Time $time, ConfigFactory $config_factory) {
    $this->currentUser = $current_user;
    $this->database = $database;
    $this->transactionRelativeManager = $transaction_relatives;
    $this->requestTime = $time->getRequestTime();
    $this->config = $config_factory->get('mcapi_signatures.settings');
  }

  /**
   * Initialise this service by telling it which transaction to work on.
   */
  public function setTransaction(TransactionInterface $transaction) : SignatureService {
    $this->transaction = $transaction;
    return $this;
  }
  /**
   * Return the transaction
   *
   * @param TransactionInterface $transaction
   * @return Transaction
   */
  public function getTransaction() : Transaction {
    return $this->transaction;
  }

  /**
   * Respond to the transaction being deleted.
   *
   * Delete from the signatory table.
   */
  public function delete() {
    $this->database->delete('mcapi_signatures')
      ->condition('serial', $this->transaction->serial->value)
      ->execute();
  }

  /**
   * Respond to the transaction being inserted.
   *
   * Write the signatories to the signatory table.
   */
  public function insert() : SignatureService {
    $q = $this->database->insert('mcapi_signatures')
      ->fields(['serial', 'uid', 'signed']);
    foreach ($this->transaction->signatures as $uid => $signed_unixtime) {
      $q->values([$this->transaction->serial->value, $uid, $signed_unixtime]);
    }
    $q->execute();
    return $this;
  }

  /**
   * Respond to the transaction being updated.
   *
   * Update the signatory table.
   */
  public function update() {
    // New signatures cannot be added, only existing ones changed.
    $serial = $this->transaction->serial->value;
    foreach ($this->transaction->signatures as $uid => $signed) {
      $this->database->update('mcapi_signatures')
        ->fields(['signed' => $signed])
        ->condition('serial', $serial)
        ->condition('uid', $uid)
        ->execute();
    }
  }

  /**
   * Add signatures to the transaction according to the configured relatives.
   */
  public function addSignatures() : SignatureService {
    $relatives = $this->transaction->type->entity
      ->getThirdPartySetting('mcapi_signatures', 'relations', []);
    $user_ids = $this->transactionRelativeManager->getUsers($this->transaction, $relatives);
    foreach ($user_ids as $uid) {
      $this->transaction->signatures[$uid] = 0;
    }
    return $this;
  }

  /**
   * Sign a transaction if the users signature is required.
   *
   * Change the state if no more signatures are left.
   *
   * @param int $uid
   *   ID of the user who is signing - defaults to the current user
   */
  public function sign($uid = NULL) {
    if (is_null($uid)) {
      $uid = $this->currentUser->id();
    }
    if (isset($this->transaction->signatures[$uid]) and empty($this->transaction->signatures[$uid])) {
      $this->transaction->signatures[$uid] = $this->requestTime;
      // Set the state to finished if there are no outstanding signatures.
      if (array_search(0, $this->transaction->signatures) === FALSE) {
        $this->transaction->set('state', 'done');
        // This should affect all the children
      }
    }
  }

  /**
   * Fulfil all remaining signatures on the transaction (admin only).
   */
  public function signOff() {
    foreach ($this->transaction->signatures as $uid => $signed) {
      if (!$signed) {
        $this->transaction->signatures[$uid] = $this->requestTime;
      }
    }
    $this->transaction->set('state', 'done');
  }

  /**
   * Determine whether the given user's signature is needed.
   *
   * @param int $uid
   *   The user ID.
   *
   * @return bool
   *   TRUE if the transaction needs the user's signature
   */
  public function isWaitingOn($uid = NULL) : bool {
    if (is_null($uid)) {
      $uid = $this->currentUser->id();
    }
    $uids = [];
    if ($this->transaction->state->target_id == 'pending') {
      foreach ($this->transaction->signatures as $user_id => $signed) {
        if ($user_id == $uid and !$signed) {
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
   * Get the transactions needing signature of a given user.
   *
   * @param int $uid
   *   The user ID.
   *
   * @return int[]
   *   Serial numbers of the transactions.
   *
   * @todo this whole module needs to handle the database better, maybe by
   * decorating the Transaction or TransactionStorage.
   */
  public static function transactionsNeedingSigOfUser($uid) : array {
    // Assumes all transactions referenced are in pending state.
    return \Drupal::database()->select("mcapi_signatures", 's')
      ->fields('s', ['serial'])
      ->condition('uid', $uid)
      ->condition('signed', 0)
      ->execute()
      ->fetchCol();
  }

}
